package jack.tutorial;


/**
 * This class introduces the main Jack features :
 *   - its integration in eclipse
 *   - how to verify a Java annotated with JML
 *   - the different views and buttons
 */

public class Intro {
	
	boolean isAnnotated;
	private AProver[] provers;
	
	/**
	 * 1. To lauch Jack on a Java annotated file, on can :
	 *    - use the "c" button
	 *    - use the contextual menu : Jack>Verify Source   
	 * @return One verification condition is generated for this method : true
	 **/
	/*@
	  @ ensures isAnnotated;
	  @*/
	void runJack() {
		isAnnotated = true;
		/* To see the verication condition, on can :
		 		- use the "e" button
		 		- use the contextual menu : Jack>Edit
		*/
	}
	
	/**
	 * 2. The project properties define :
	 * 		- the JML Class Path
	 * 		- the JML clauses default values
	 * 		- the default classes serialized image
	 * @param p : The project properties can be acceded from the contextual menu
	 * of the project : Properties>Jack 
	 **/
	/*@
	  @ requires p != null;
	  @*/
	void setProjectProperties(ProjectProperties p) {
		/*
		 * 2.1 By default, the Java class path is used to perform the link.
		 * One may need to use annotated APIs, in that case, one can define
		 * a JML class path.
		 * @see annotatedAPIsAreNeeded()
		 */
		if (annotatedAPIsAreNeeded()) {
			p.setJmlPath();
		}
		
		/*
		 * 2.2 One can change the default value of JML clauses
		 * @see ProjectProperties.setDefaultBehaviour()
		 */
		p.setDefaultBehaviour();
		
		/*
		 * 2.3 Generating a serialized image of the default classes allow to improve 
		 * in time the verification generation
		 * @see ProjectProperties.setClassesToSerialized()
		 */
		p.setClassesToSerialized();
		Util.generateImage(p);
	}

	/**
	 * 2.1 One can need an annotated API, here if one want to prove that this method
	 * does not throw any runtime exception, the class System has to be annotated.
	 * This mean that one have to write a JML specification for the class and to add
	 * the path to the JML class path.
	 */
	//@ exsures (RuntimeException) false;
	private boolean annotatedAPIsAreNeeded() {
		String s = "Do you need annotated API ?";
		System.out.println(s);
		try {
		return System.in.read() == 0;
		}
		catch (Exception ioe) {
			return false;
		}
	}

	/**
	 * 3. The Jack preferences allow to :
	 *  - set preferences for compiler
	 *  - set preferences for the lemma viewer
	 *  - set preferences for each prover
	 **/
	/*@ ensures (\forall int i; 0 <= i && i < provers.length; provers[i].arePreferencesSet);*/
	void setPreferences(Preferences p) {
		// @see Preferences.setCompilerPreferences()
		p.setCompilerPreferences();
		
		// @see Preferences.setLemmaViewerPreferences()
		p.setLemmaViewerPreferences();
		
		for (int i=0; i < provers.length; i++)
			// each prover plugin can define its preferences
			// @see Prover.setProverPreferences(Preferences)
			provers[i].setProverPreferences(p);
	}
}
